<?php

namespace Telnyx;

/**
 * Class AvailablePhoneNumber
 *
 * @package Telnyx
 */
class AvailablePhoneNumber extends ApiResource
{
    const OBJECT_NAME = "available_phone_number";

    use ApiOperations\All;
}
