<?php
/**
 * @copyright 2021 Double Bastion LLC <www.doublebastion.com>
 *
 * @author Double Bastion LLC
 *
 * @license GNU AGPL version 3 or any later version
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU AFFERO GENERAL PUBLIC LICENSE for more details.
 *
 * You should have received a copy of the GNU Affero General Public
 * License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

declare(strict_types=1);

script('sms_relentless', 'adminsettings');
style('sms_relentless', 'style');

?>

<div id="sms_relentless_adm">

        <div class="section">
           <h2><?php p($l->t('SMS Relentless'));?></h2>
           <p id="settings-dscr"><h3><?php p($l->t('Enter your options in the fields from below:')); ?></h3></p><br>
           <div class="sms_followupsection">

                <form id="admin_conf" action="#" >

                   <p><label for="showallmessages" class="setlabeltext"><b>In my message tables, list all the sent/received messages of all the users, not only my messages:</b></label></p>
                   <input type="checkbox" class="sms_rel_checkbox" id="showallmessages" name="showallmessages" autocomplete="false" /><br>

                   <p><label for="restrNmbrAccess" class="setlabeltext"><b>Restrict access to the following phone numbers:</b></label></p><br>

                   <table id="phoneNmbrRestr">

                      <tr><th>Restriction Author</th><th id="phoneNumberTh">Phone Number</th><th>Groups Allowed</th><th style="max-width: 40px !important"></th><th>Users Allowed</th><th style="max-width: 40px !important"></th><th></th><th></th></tr>

                   </table><br>

                   <input type="hidden" id="user_id" name="user_id" />

                   <input id="save_admin_settings" type="submit" value="<?php p($l->t('Save')) ?>" />

                   <span id="smsr_save_msg"></span>

               </form>
           </div>
        </div>
</div>

