<?php

namespace Telnyx;

/**
 * Class Verification
 *
 * @package Telnyx
 */
class VerifyVerification extends ApiResource
{
    const OBJECT_NAME = "verify_verification";

    // Note: this is only a result used by VerificationTest
}