<?php
/**
 * @copyright 2021 Double Bastion LLC <www.doublebastion.com>
 *
 * @author Double Bastion LLC
 *
 * @license GNU AGPL version 3 or any later version
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU AFFERO GENERAL PUBLIC LICENSE for more details.
 *
 * You should have received a copy of the GNU Affero General Public
 * License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

declare(strict_types=1);

return [
  'routes' => [
     // User views
     ['name' => 'page#index', 'url' => '/', 'verb' => 'GET'],
     ['name' => 'author_api#receivesmstel', 'url' => '/api/recsmstel', 'verb' => 'POST'],
     ['name' => 'author_api#receivesmsnex', 'url' => '/api/recsmsnex', 'verb' => 'POST'],
     ['name' => 'author_api#receivesmstwil', 'url' => '/api/recsmstwil', 'verb' => 'POST'],
     ['name' => 'author_api#receivesmsflow', 'url' => '/api/recsmsflow', 'verb' => 'POST'],
     ['name' => 'author_api#recdeliveryreptel', 'url' => '/api/recdeliverytel', 'verb' => 'POST'],
     ['name' => 'author_api#recdeliveryrepnex', 'url' => '/api/recdeliverynex', 'verb' => 'POST'],
     ['name' => 'author_api#recdeliveryreptwil', 'url' => '/api/recdeliverytwil', 'verb' => 'POST'],
     ['name' => 'author_api#recdeliveryrepflow', 'url' => '/api/recdeliveryflow', 'verb' => 'POST'],
     ['name' => 'smsrelentless#cleantempdir', 'url' => '/user/cleantempdir/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#getreceivedtable', 'url' => '/user/getreceivedtable/{userId}', 'verb' => 'GET'],
     ['name' => 'smsrelentless#getreceivedtablefordel', 'url' => '/user/getreceivedtablefordel/{userId}', 'verb' => 'GET'],
     ['name' => 'smsrelentless#getsenttable', 'url' => '/user/getsenttable/{userId}', 'verb' => 'GET'],
     ['name' => 'smsrelentless#getsenttablefordel', 'url' => '/user/getsenttablefordel/{userId}', 'verb' => 'GET'],
     ['name' => 'smsrelentless#getgroupedtable', 'url' => '/user/getgroupedtable/{userId}', 'verb' => 'GET'],
     ['name' => 'smsrelentless#getgroupedpernumber', 'url' => '/user/getgroupedpernumber/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#removerecrows', 'url' => '/user/removerecrows/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#removesentrows', 'url' => '/user/removesentrows/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#saveoldrecrows', 'url' => '/user/saveoldrecrows/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#saveoldsentrows', 'url' => '/user/saveoldsentrows/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#getbalancetel', 'url' => '/user/getbalancetel/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#getbalancenex', 'url' => '/user/getbalancenex/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#getbalancetwil', 'url' => '/user/getbalancetwil/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#getbalanceflow', 'url' => '/user/getbalanceflow/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#getsmsnumbers', 'url' => '/user/getsmsnumbers/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#upload_numbers_file', 'url' => '/user/upload_numbers_file/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#uploadfile', 'url' => '/user/uploadfile/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#pickfile', 'url' => '/user/pickfile/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#removeupfile', 'url' => '/user/removeupfile/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#sendsmstel', 'url' => '/user/sendsmstel/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#sendsmsnex', 'url' => '/user/sendsmsnex/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#sendsmstwil', 'url' => '/user/sendsmstwil/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#sendsmsflow', 'url' => '/user/sendsmsflow/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#getmessagesperpage', 'url' => '/user/getmessagesperpage/{userId}', 'verb' => 'GET'],
     ['name' => 'smsrelentless#savedisplayname', 'url' => '/user/savedisplayname/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#getautoreplyconf', 'url' => '/user/getautoreplyconf/{userId}', 'verb' => 'GET'],
     // Settings
     ['name' => 'smsrelentless#getsettings', 'url' => '/user/getsettings/{userId}', 'verb' => 'GET'],
     ['name' => 'smsrelentless#getadminsettings', 'url' => '/user/getadminsettings/{userId}', 'verb' => 'GET'],
     ['name' => 'smsrelentless#updatesettings', 'url' => '/user/updatesettings/{userId}', 'verb' => 'PUT'],
     ['name' => 'smsrelentless#updateadminsettings', 'url' => '/user/updateadminsettings/{userId}', 'verb' => 'PUT'],
     ['name' => 'smsrelentless#updatenumberrestrictions', 'url' => '/user/updatenumberrestrictions/{userId}', 'verb' => 'PUT'],
     ['name' => 'smsrelentless#updateautoreplies', 'url' => '/user/updateautoreplies/{userId}', 'verb' => 'PUT'],
     ['name' => 'smsrelentless#removenumberrestrictions', 'url' => '/user/removenumberrestrictions/{userId}', 'verb' => 'POST'],
     ['name' => 'smsrelentless#removeautoreplies', 'url' => '/user/removeautoreplies/{userId}', 'verb' => 'POST'],
  ]
];

